"use strict";

const UglifyJS = require('uglify-js');
const UglifyJSHarmony = require('uglify-es');
const process = require('process');

const JS_MINIFY_ERR_CODE_CONTEXT_SIZE = 800;

module.exports = function(code, settings) {
	code = code.trim();
	if (!code) {
		return code;
	}

	let compressSettings = {
		sequences: true,
		properties: true,
		dead_code: true,
		drop_debugger: !settings.debug,
		unsafe: true,
		conditionals: true,
		comparisons: true,
		evaluate: true,
		booleans: true,
		loops: true,
		unused: true,
		hoist_funs: true,
		hoist_vars: false,
		if_return: true,
		join_vars: true,
		cascade: true,
		collapse_vars: true,
		reduce_vars: true,
		warnings: true,
		negate_iife: true,
		drop_console: !settings.debug,
		keep_fargs: true,
		keep_fnames: false,
		passes: settings.passes || 1,
	};

	try {
		return (settings.harmony ? UglifyJSHarmony : UglifyJS).minify(code, {
			warnings: true,
			fromString: true,
			mangle: {
				eval: true,
			},
			compress: compressSettings,
		}).code;
	} catch (e) {
		let position = e.pos || 0;
		let area = JS_MINIFY_ERR_CODE_CONTEXT_SIZE;
		let startPos = Math.max(0, e.pos - area / 2);
		let leftArea = position - startPos;
		let endPos = position + (area - leftArea);
		// Use console.error as UglifyJS throws plain objects, not Error instances
		console.error(e);
		console.log("\n========== CODE BLOCK (surrounding 800 characters) ==========\n");
		console.info(code.slice(startPos, endPos + 1));
		process.exit(1);
	}
};
